package Alien::Libarchive::MSWin32;

use strict;
use warnings;
use base qw( Alien::Libarchive );

# ABSTRACT: Build and make available libarchive on Windows
our $VERSION = '0.20'; # VERSION


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Alien::Libarchive::MSWin32 - Build and make available libarchive on Windows

=head1 VERSION

version 0.20

=head1 DESCRIPTION

This module is deprecated and kept only for backwards
compatibility.  You should use L<Alien::Libarchive>
instead.

=head1 SEE ALSO

=over 4

=item L<Alien::Libarchive>

=item L<Archive::Libarchive::XS>

=item L<Archive::Libarchive::FFI>

=item L<Archive::Libarchive::Any>

=item L<Archive::Ar::Libarchive>

=item L<Archive::Peek::Libarchive>

=item L<Archive::Extract::Libarchive>

=back

=head1 AUTHOR

Graham Ollis <plicease@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Graham Ollis.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
